// 2012 (c) daniel.sangorrin@gmail.com
// Test for the ATT_TER and the SGI (software generated interrupts)
// Expected output:
// Processor 2 start.
// Processor 1 start.
// Main2 starts.
// Simple starts (exinf = 1).
// HANDLER 1
// HANDLER 2
// Main1 ends.
// Last message 2
// Last message 1
#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include <sil.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "sample1.h"

/*
 *  サービスコールのエラーのログ出力
 */
Inline void
svc_perror(const char *file, int_t line, const char *expr, ER ercd)
{
        if (ercd < 0) {
                t_perror(LOG_ERROR, file, line, expr, ercd);
        }
}

#define SVC_PERROR(expr)   svc_perror(__FILE__, __LINE__, #expr, (expr))

/*
 * Last messages
 */
void last_message_1(intptr_t exinf)
{
        syslog(LOG_EMERG, "Last message 1");
}

void last_message_2(intptr_t exinf)
{
        syslog(LOG_EMERG, "Last message 2");
}

/*
 * SGI handlers
 */
void my_sgi_handler_1(void)
{
        syslog(LOG_EMERG, "HANDLER 1");
}

void my_sgi_handler_2(void)
{
        syslog(LOG_EMERG, "HANDLER 2");
}

/*
 * Main task
 */
void main_task(intptr_t exinf)
{
        ER_UINT ercd;

        SVC_PERROR(syslog_msk_log(0, LOG_UPTO(LOG_DEBUG)));
        syslog(LOG_NOTICE, "Simple starts (exinf = %d).", (int_t) exinf);

        ercd = serial_opn_por(TASK_PORTID_G_SYSLOG);
        if (ercd < 0 && MERCD(ercd) != E_OBJ) {
                syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
                       itron_strerror(ercd), SERCD(ercd));
        }
        SVC_PERROR(serial_ctl_por(TASK_PORTID_G_SYSLOG,
                                  (IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

        dic_swi((1<<0), 6); // SGI to myself
        dic_swi((1<<1), 6); // SGI to core2

        SVC_PERROR(act_tsk(MAIN_TASK2));
        syslog(LOG_NOTICE, "Main1 ends.");

        SVC_PERROR(ext_ker());
        assert(0);
}

void main_task_2(intptr_t exinf)
{
        syslog(LOG_NOTICE, "Main2 starts.");
        while(1);
}