#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include <sil.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "sample1.h"

/*
 *  サービスコールのエラーのログ出力
 */
Inline void
svc_perror(const char *file, int_t line, const char *expr, ER ercd)
{
        if (ercd < 0) {
                t_perror(LOG_ERROR, file, line, expr, ercd);
        }
}

#define SVC_PERROR(expr)        svc_perror(__FILE__, __LINE__, #expr, (expr))

/*
 * Timestamp function
 */
static uint32_t __div64_32(uint64_t *n, uint32_t base)
{
        uint64_t rem = *n;
        uint64_t b = base;
        uint64_t res, d = 1;
        uint32_t high = rem >> 32;

        /* Reduce the thing a bit first */
        res = 0;
        if (high >= base) {
                high /= base;
                res = (uint64_t) high << 32;
                rem -= (uint64_t) (high*base) << 32;
        }

        while ((int64_t)b > 0 && b < rem) {
                b = b+b;
                d = d+d;
        }

        do {
                if (rem >= b) {
                        rem -= b;
                        res += d;
                }
                b >>= 1;
                d >>= 1;
        } while (d);

        *n = res;
        return rem;
}

#define do_div(n,base) ({                              \
       uint32_t __base = (base);                       \
       uint32_t __rem;                                 \
       (void)(((typeof((n)) *)0) == ((uint64_t *)0));  \
       if (((n) >> 32) == 0) {                         \
               __rem = (uint32_t)(n) % __base;         \
               (n) = (uint32_t)(n) / __base;           \
       } else                                          \
               __rem = __div64_32(&(n), __base);       \
       __rem;                                          \
})

static uint64_t global_counter_get_cycles()
{
        PERFCNT cycles;
        x_get_pcc(&cycles);
        return x_cnv_nsec(cycles);
}

#define NSEC_PER_SEC  1000000000L

static void get_timestamp(uint32_t *s, uint32_t *us)
{
        uint64_t ts = global_counter_get_cycles();
        *us = do_div(ts, NSEC_PER_SEC);
        *us /= 1000;
        *s = (uint32_t)ts;
}

/*
 * Periodic task
 */
void task(intptr_t exinf)
{
        static int i = 0;
        uint32_t s, us;

        if (i!=0) {
                get_timestamp(&s, &us);
                syslog(LOG_NOTICE, "task %d: [%5lu.%06lu]", i, s, us);
        }

        i++;
        x_rst_pcc();
}

/*
 * Cyclic handler
 */
void cyclic_handler(intptr_t exinf)
{
        SVC_PERROR(iact_tsk(TASK1_1));
}

/*
 * Main task
 */

void main_task(intptr_t exinf)
{
        ER_UINT ercd;

        SVC_PERROR(syslog_msk_log(0, LOG_UPTO(LOG_DEBUG)));
        syslog(LOG_NOTICE, "Simple starts (exinf = %d).", (int_t) exinf);

        ercd = serial_opn_por(TASK_PORTID_G_SYSLOG);
        if (ercd < 0 && MERCD(ercd) != E_OBJ) {
                syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
                       itron_strerror(ercd), SERCD(ercd));
        }
        SVC_PERROR(serial_ctl_por(TASK_PORTID_G_SYSLOG,
                                  (IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

        x_init_pcc(); // Init the global counter for timestamping

        SVC_PERROR(sta_cyc(CYCHDR1)); // start the cyclic handler

        syslog(LOG_NOTICE, "Main goes to sleep.");
        SVC_PERROR(slp_tsk());
        SVC_PERROR(ext_ker());
        assert(0);
}
