#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include <sil.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "sample1.h"

/*
 *  サービスコールのエラーのログ出力
 */
Inline void
svc_perror(const char *file, int_t line, const char *expr, ER ercd)
{
        if (ercd < 0) {
                t_perror(LOG_ERROR, file, line, expr, ercd);
        }
}

#define SVC_PERROR(expr)        svc_perror(__FILE__, __LINE__, #expr, (expr))

#define NO_LOOP         ULONG_C(1000000)
static SYSTIM  empty_time;

static void
test_empty(void)
{
        SYSTIM  stime, etime;
        volatile ulong_t        i;

        get_tim(&stime);
        for (i = 0; i < NO_LOOP; i++) {
        }
        get_tim(&etime);
        empty_time = etime - stime;
        syslog(LOG_NOTICE, "empty loop: %u", empty_time);
}

static void
test_dly_nse(ulong_t dlytim)
{
        SYSTIM  stime, etime, delay_time;
        volatile ulong_t        i;

        get_tim(&stime);
        for (i = 0; i < NO_LOOP; i++) {
                sil_dly_nse(dlytim);
        }
        get_tim(&etime);
        delay_time = (etime - stime) - empty_time;
        syslog(LOG_NOTICE, "sil_dly_nse(%u): %u %s", (uint_t)(dlytim),
                           (uint_t)(delay_time), delay_time > dlytim ? "OK" : "NG");
}

void
test_body(void)
{
        test_empty();

        syslog(LOG_NOTICE, "-- for fitting parameters --");
        test_dly_nse(0);
        test_dly_nse(SIL_DLY_TIM1);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 1);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 2);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 3);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 4);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 5);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 10);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 20);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 50);

        syslog(LOG_NOTICE, "-- for checking boundary conditions --");
        test_dly_nse(SIL_DLY_TIM1 + 1);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 1 + 1);
        test_dly_nse(SIL_DLY_TIM1 + SIL_DLY_TIM2 * 2 + 1);

        syslog(LOG_NOTICE, "Finish");
}

/*
 * Main task
 */
void main_task(intptr_t exinf)
{
        ER_UINT ercd;

        SVC_PERROR(syslog_msk_log(0, LOG_UPTO(LOG_DEBUG)));
        syslog(LOG_NOTICE, "Simple starts (exinf = %d).", (int_t) exinf);

        ercd = serial_opn_por(TASK_PORTID_G_SYSLOG);
        if (ercd < 0 && MERCD(ercd) != E_OBJ) {
                syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
                       itron_strerror(ercd), SERCD(ercd));
        }
        SVC_PERROR(serial_ctl_por(TASK_PORTID_G_SYSLOG,
                                  (IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

        test_body();

        SVC_PERROR(ext_ker());
        assert(0);
}
